@@ -26,11 +26,6 @@
 #ifndef PSYCOPG_TYPECAST_H
 #define PSYCOPG_TYPECAST_H 1
 
-#define PY_SSIZE_T_CLEAN
-#include <Python.h>
-
-#include "psycopg/config.h"
-
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -82,17 +77,14 @@ HIDDEN int typecast_add(PyObject *obj, PyObject *dict, int binary);
 /* the C callable typecastObject creator function */
 HIDDEN PyObject *typecast_from_c(typecastObject_initlist *type, PyObject *d);
 
-/* the python callable typecast creator function */
+/* the python callable typecast creator functions */
 HIDDEN PyObject *typecast_from_python(
     PyObject *self, PyObject *args, PyObject *keywds);
+HIDDEN PyObject *typecast_array_from_python(
+    PyObject *self, PyObject *args, PyObject *keywds);
 
 /* the function used to dispatch typecasting calls */
 HIDDEN PyObject *typecast_cast(
     PyObject *self, const char *str, Py_ssize_t len, PyObject *curs);
 
-/** utility functions **/
-
-HIDDEN const char *skip_until_space(const char *s);
-HIDDEN const char *skip_until_space2(const char *s, Py_ssize_t *len);
-
 #endif /* !defined(PSYCOPG_TYPECAST_H) */
