@@ -1,10 +1,32 @@
 #!/usr/bin/env python
 
+# test_dates.py - unit test for dates handling
+#
+# Copyright (C) 2008-2011 James Henstridge  <james@jamesh.id.au>
+#
+# psycopg2 is free software: you can redistribute it and/or modify it
+# under the terms of the GNU Lesser General Public License as published
+# by the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# In addition, as a special exception, the copyright holders give
+# permission to link this program with the OpenSSL library (or with
+# modified versions of OpenSSL that use the same license as OpenSSL),
+# and distribute linked combinations including the two.
+#
+# You must obey the GNU Lesser General Public License in all respects for
+# all of the code used other than OpenSSL.
+#
+# psycopg2 is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+# License for more details.
+
 import math
 import unittest
-import tests
 import psycopg2
 from psycopg2.tz import FixedOffsetTimezone
+from testconfig import dsn
 
 class CommonDatetimeTestsMixin:
 
@@ -14,7 +36,7 @@ class CommonDatetimeTestsMixin:
 
     def test_parse_date(self):
         value = self.DATE('2007-01-01', self.curs)
-        self.assertNotEqual(value, None)
+        self.assert_(value is not None)
         self.assertEqual(value.year, 2007)
         self.assertEqual(value.month, 1)
         self.assertEqual(value.day, 1)
@@ -29,7 +51,7 @@ class CommonDatetimeTestsMixin:
 
     def test_parse_time(self):
         value = self.TIME('13:30:29', self.curs)
-        self.assertNotEqual(value, None)
+        self.assert_(value is not None)
         self.assertEqual(value.hour, 13)
         self.assertEqual(value.minute, 30)
         self.assertEqual(value.second, 29)
@@ -44,7 +66,7 @@ class CommonDatetimeTestsMixin:
 
     def test_parse_datetime(self):
         value = self.DATETIME('2007-01-01 13:30:29', self.curs)
-        self.assertNotEqual(value, None)
+        self.assert_(value is not None)
         self.assertEqual(value.year, 2007)
         self.assertEqual(value.month, 1)
         self.assertEqual(value.day, 1)
@@ -75,12 +97,12 @@ class DatetimeTests(unittest.TestCase, CommonDatetimeTestsMixin):
     """Tests for the datetime based date handling in psycopg2."""
 
     def setUp(self):
-        self.conn = psycopg2.connect(tests.dsn)
+        self.conn = psycopg2.connect(dsn)
         self.curs = self.conn.cursor()
-        self.DATE = psycopg2._psycopg.PYDATE
-        self.TIME = psycopg2._psycopg.PYTIME
-        self.DATETIME = psycopg2._psycopg.PYDATETIME
-        self.INTERVAL = psycopg2._psycopg.PYINTERVAL
+        self.DATE = psycopg2.extensions.PYDATE
+        self.TIME = psycopg2.extensions.PYTIME
+        self.DATETIME = psycopg2.extensions.PYDATETIME
+        self.INTERVAL = psycopg2.extensions.PYINTERVAL
 
     def tearDown(self):
         self.conn.close()
@@ -243,18 +265,18 @@ class DatetimeTests(unittest.TestCase, CommonDatetimeTestsMixin):
 
     def test_type_roundtrip_date(self):
         from datetime import date
-        self._test_type_roundtrip(date(2010,05,03))
+        self._test_type_roundtrip(date(2010,5,3))
 
     def test_type_roundtrip_datetime(self):
         from datetime import datetime
-        dt = self._test_type_roundtrip(datetime(2010,05,03,10,20,30))
+        dt = self._test_type_roundtrip(datetime(2010,5,3,10,20,30))
         self.assertEqual(None, dt.tzinfo)
 
     def test_type_roundtrip_datetimetz(self):
         from datetime import datetime
         import psycopg2.tz
         tz = psycopg2.tz.FixedOffsetTimezone(8*60)
-        dt1 = datetime(2010,05,03,10,20,30, tzinfo=tz)
+        dt1 = datetime(2010,5,3,10,20,30, tzinfo=tz)
         dt2 = self._test_type_roundtrip(dt1)
         self.assertNotEqual(None, dt2.tzinfo)
         self.assertEqual(dt1, dt2)
@@ -269,11 +291,11 @@ class DatetimeTests(unittest.TestCase, CommonDatetimeTestsMixin):
 
     def test_type_roundtrip_date_array(self):
         from datetime import date
-        self._test_type_roundtrip_array(date(2010,05,03))
+        self._test_type_roundtrip_array(date(2010,5,3))
 
     def test_type_roundtrip_datetime_array(self):
         from datetime import datetime
-        self._test_type_roundtrip_array(datetime(2010,05,03,10,20,30))
+        self._test_type_roundtrip_array(datetime(2010,5,3,10,20,30))
 
     def test_type_roundtrip_time_array(self):
         from datetime import time
@@ -285,7 +307,7 @@ class DatetimeTests(unittest.TestCase, CommonDatetimeTestsMixin):
 
 
 # Only run the datetime tests if psycopg was compiled with support.
-if not hasattr(psycopg2._psycopg, 'PYDATETIME'):
+if not hasattr(psycopg2.extensions, 'PYDATETIME'):
     del DatetimeTests
 
 
@@ -293,7 +315,7 @@ class mxDateTimeTests(unittest.TestCase, CommonDatetimeTestsMixin):
     """Tests for the mx.DateTime based date handling in psycopg2."""
 
     def setUp(self):
-        self.conn = psycopg2.connect(tests.dsn)
+        self.conn = psycopg2.connect(dsn)
         self.curs = self.conn.cursor()
         self.DATE = psycopg2._psycopg.MXDATE
         self.TIME = psycopg2._psycopg.MXTIME
@@ -314,7 +336,7 @@ class mxDateTimeTests(unittest.TestCase, CommonDatetimeTestsMixin):
 
     def test_parse_bc_date(self):
         value = self.DATE('00042-01-01 BC', self.curs)
-        self.assertNotEqual(value, None)
+        self.assert_(value is not None)
         # mx.DateTime numbers BC dates from 0 rather than 1.
         self.assertEqual(value.year, -41)
         self.assertEqual(value.month, 1)
@@ -322,7 +344,7 @@ class mxDateTimeTests(unittest.TestCase, CommonDatetimeTestsMixin):
 
     def test_parse_bc_datetime(self):
         value = self.DATETIME('00042-01-01 13:30:29 BC', self.curs)
-        self.assertNotEqual(value, None)
+        self.assert_(value is not None)
         # mx.DateTime numbers BC dates from 0 rather than 1.
         self.assertEqual(value.year, -41)
         self.assertEqual(value.month, 1)
@@ -373,7 +395,7 @@ class mxDateTimeTests(unittest.TestCase, CommonDatetimeTestsMixin):
 
     def test_parse_interval(self):
         value = self.INTERVAL('42 days 05:50:05', self.curs)
-        self.assertNotEqual(value, None)
+        self.assert_(value is not None)
         self.assertEqual(value.day, 42)
         self.assertEqual(value.hour, 5)
         self.assertEqual(value.minute, 50)
@@ -395,7 +417,11 @@ class mxDateTimeTests(unittest.TestCase, CommonDatetimeTestsMixin):
         from mx.DateTime import DateTime
         value = self.execute('select (%s)::timestamp::text',
                              [DateTime(-41, 1, 1, 13, 30, 29.123456)])
-        self.assertEqual(value, '0042-01-01 13:30:29.123456 BC')
+        # microsecs for BC timestamps look not available in PG < 8.4
+        # but more likely it's determined at compile time.
+        self.assert_(value in (
+            '0042-01-01 13:30:29.123456 BC',
+            '0042-01-01 13:30:29 BC'), value)
 
     def test_adapt_timedelta(self):
         from mx.DateTime import DateTimeDeltaFrom
@@ -426,11 +452,11 @@ class mxDateTimeTests(unittest.TestCase, CommonDatetimeTestsMixin):
 
     def test_type_roundtrip_date(self):
         from mx.DateTime import Date
-        self._test_type_roundtrip(Date(2010,05,03))
+        self._test_type_roundtrip(Date(2010,5,3))
 
     def test_type_roundtrip_datetime(self):
         from mx.DateTime import DateTime
-        self._test_type_roundtrip(DateTime(2010,05,03,10,20,30))
+        self._test_type_roundtrip(DateTime(2010,5,3,10,20,30))
 
     def test_type_roundtrip_time(self):
         from mx.DateTime import Time
@@ -442,11 +468,11 @@ class mxDateTimeTests(unittest.TestCase, CommonDatetimeTestsMixin):
 
     def test_type_roundtrip_date_array(self):
         from mx.DateTime import Date
-        self._test_type_roundtrip_array(Date(2010,05,03))
+        self._test_type_roundtrip_array(Date(2010,5,3))
 
     def test_type_roundtrip_datetime_array(self):
         from mx.DateTime import DateTime
-        self._test_type_roundtrip_array(DateTime(2010,05,03,10,20,30))
+        self._test_type_roundtrip_array(DateTime(2010,5,3,10,20,30))
 
     def test_type_roundtrip_time_array(self):
         from mx.DateTime import Time
@@ -458,7 +484,10 @@ class mxDateTimeTests(unittest.TestCase, CommonDatetimeTestsMixin):
 
 
 # Only run the mx.DateTime tests if psycopg was compiled with support.
-if not hasattr(psycopg2._psycopg, 'MXDATETIME'):
+try:
+    if not hasattr(psycopg2._psycopg, 'MXDATETIME'):
+        del mxDateTimeTests
+except AttributeError:
     del mxDateTimeTests
 
 
